# SQL.pm
#
# This module provides a class to represent SQL objects
# for SQL::Interpolate.  SQL objects map 1-1 to quote-like
# sql// operators with source filtering enabled.
#
# Copyright (c) 2003 David Manura
# This file is licensed under the same terms as Perl itself.

package SQL::Interpolate::SQL;
use strict;
use base qw(Exporter);
use overload '.'  => \&concat,
             '""' => \&stringify;

sub new
{
    my($class,
       @list   # list of SQL string, variable references,
               #   macro objects, and SQL objects.
    ) = @_;
    my $self = {
        list => \@list
    };
    bless $self, $class;
    return $self;
}

# Concatenate SQL object with another expression.
# An SQL object can be concatenated with another SQL object,
# variable reference, or and SQL string.
sub concat
{
    my($a, $b, $inverted) = @_;
    my $c = $inverted
        ? new SQL::Interpolate::SQL(
            ref($b) eq __PACKAGE__ ? @{$b->{list}} : ($b),
            @{$a->{list}}
          )
        : new SQL::Interpolate::SQL(
            @{$a->{list}},
            ref($b) eq __PACKAGE__ ? @{$b->{list}} : ($b)
          )
    ;

    return $c;
}

sub stringify
{
    my($a) = @_;
    return $a;
}

1
